@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager.c,v $
- * $Date: 2010/03/22 14:47:53 $
- * $Revision: 1.42.2.5 $
+ * $Date: 2010/07/01 19:18:09 $
+ * $Revision: 1.42.2.7 $
  * $Author: bester $
  */
 
@@ -753,6 +753,7 @@ globus_l_gram_job_manager_add_ref_stub(
     (*ref)->exit_code = request->exit_code;
     (*ref)->status_count = 0;
     (*ref)->loaded_only = GLOBUS_FALSE;
+    (*ref)->seg_last_timestamp = request ? request->seg_last_timestamp : 0;
 
     (*ref)->key = strdup(key);
     if ((*ref)->key == NULL)
@@ -1029,12 +1030,16 @@ globus_l_gram_job_manager_remove_reference_locked(
              */
             request = ref->request;
 
-            /* If the request is complete we can destroy it
+            /* If the request is complete, or we are shutting down due to
+             * a proxy expire, we can destroy it
              */
-            if (request->jobmanager_state ==
+            if ((request->jobmanager_state ==
                     GLOBUS_GRAM_JOB_MANAGER_STATE_DONE ||
                 request->jobmanager_state ==
-                    GLOBUS_GRAM_JOB_MANAGER_STATE_FAILED_DONE)
+                    GLOBUS_GRAM_JOB_MANAGER_STATE_FAILED_DONE) ||
+                (request->jobmanager_state ==
+                    GLOBUS_GRAM_JOB_MANAGER_STATE_STOP &&
+                        (manager->stop == GLOBUS_TRUE)))
             {
                 globus_gram_job_manager_log(
                         manager,
@@ -1084,7 +1089,15 @@ globus_l_gram_job_manager_remove_reference_locked(
                     }
                 }
 
-                globus_gram_job_manager_request_destroy(ref->request);
+                if (ref->request->jobmanager_state ==
+                            GLOBUS_GRAM_JOB_MANAGER_STATE_STOP)
+                {
+                    globus_gram_job_manager_request_free(ref->request);
+                }
+                else
+                {
+                    globus_gram_job_manager_request_destroy(ref->request);
+                }
                 free(ref->request);
                 free(ref->key);
                 free(ref);
@@ -1209,7 +1222,8 @@ globus_gram_job_manager_register_job_id(
             request->job_contact_path,
             job_id);
 
-    if (manager->config->seg_module != NULL)
+    if (manager->config->seg_module != NULL || 
+        strcmp(manager->config->jobmanager_type, "condor") == 0)
     {
         /* If we're using the SEG, split on /,/ so that seg events can be
          * matched to the relevant job requests
@@ -2260,7 +2274,8 @@ globus_gram_job_manager_request_load_all(
              * the job's LRM job id 
              */
             if ((request->config->seg_module != NULL ||
-                 strcmp(request->config->jobmanager_type, "fork") == 0) &&
+                 strcmp(request->config->jobmanager_type, "fork") == 0 ||
+                 strcmp(request->config->jobmanager_type, "condor") == 0) &&
                 (request->restart_state ==
                         GLOBUS_GRAM_JOB_MANAGER_STATE_POLL1 ||
                  request->restart_state ==
