@@ -22,8 +22,8 @@
  *
  * CVS Information:
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager_validate.c,v $
- * $Date: 2010/03/22 14:47:53 $
- * $Revision: 1.9.6.2 $
+ * $Date: 2010/06/03 17:45:57 $
+ * $Revision: 1.9.6.5 $
  * $Author: bester $
  */
 
@@ -102,7 +102,10 @@ globus_l_gram_job_manager_validation_rsl_error(
 static
 int
 globus_l_gram_job_manager_validation_value_error(
-    const char *                        attribute);
+    globus_gram_jobmanager_request_t *  request,
+    const char *                        attribute,
+    const char *                        value,
+    const char *                        enumerated_values);
 
 static
 int
@@ -168,7 +171,7 @@ globus_gram_job_manager_validation_init(
                 scheduler_validation_filename);
     }
 
-    if (manager->config->log_levels & GLOBUS_GRAM_JOB_MANAGER_LOG_DEBUG)
+    if (manager->config->log_levels & GLOBUS_GRAM_JOB_MANAGER_LOG_TRACE)
     {
         tmp_list = manager->validation_records;
 
@@ -190,9 +193,9 @@ globus_gram_job_manager_validation_init(
 
             globus_gram_job_manager_log(
                     manager,
-                    GLOBUS_GRAM_JOB_MANAGER_LOG_DEBUG,
+                    GLOBUS_GRAM_JOB_MANAGER_LOG_TRACE,
                     "event=gram.validation_record.info "
-                    "level=DEBUG "
+                    "level=TRACE "
                     "attribute=%s "
                     "description=\"%s\" "
                     "required_when=%d "
@@ -992,7 +995,7 @@ globus_l_gram_job_manager_check_rsl_attributes(
                     "status=%d "
                     "\n",
                     globus_rsl_relation_get_attribute(relation),
-                    when,
+                    whenstr,
                     -rc);
             if (request->gt3_failure_message == NULL)
             {
@@ -1022,7 +1025,10 @@ globus_l_gram_job_manager_check_rsl_attributes(
             if(strstr(record->enumerated_values, value_str) == GLOBUS_NULL)
             {
                 rc = globus_l_gram_job_manager_validation_value_error(
-                            attribute);
+                            request,
+                            attribute,
+                            value_str,
+                            record->enumerated_values);
 
                 globus_gram_job_manager_request_log(
                         request,
@@ -1100,9 +1106,9 @@ globus_l_gram_job_manager_insert_default_rsl(
 
                 globus_gram_job_manager_request_log(
                         request,
-                        GLOBUS_GRAM_JOB_MANAGER_LOG_DEBUG,
+                        GLOBUS_GRAM_JOB_MANAGER_LOG_TRACE,
                         "event=gram.validate_rsl.info "
-                        "level=DEBUG "
+                        "level=TRACE "
                         "msg=\"Inserting default RSL for attribute\" "
                         "attribute=%s "
                         "default=\"%s\" "
@@ -1335,8 +1341,20 @@ globus_l_gram_job_manager_validation_rsl_error(
 static
 int
 globus_l_gram_job_manager_validation_value_error(
-    const char *                        attribute)
+    globus_gram_jobmanager_request_t *  request,
+    const char *                        attribute,
+    const char *                        value,
+    const char *                        enumerated_values)
 {
+    if (request->gt3_failure_message == NULL)
+    {
+        request->gt3_failure_message = globus_common_create_string(
+                "RSL attribute \"%s\" has value \"%s\" which is not one of the allowed values (%s)",
+                attribute,
+                value,
+                enumerated_values);
+    }
+
     HANDLE_RSL_ERROR(GLOBUS_GRAM_PROTOCOL_COUNT_PARAM,
                      GLOBUS_GRAM_PROTOCOL_ERROR_INVALID_COUNT)
     HANDLE_RSL_ERROR(GLOBUS_GRAM_PROTOCOL_MYJOB_PARAM,
